/***************************************************************************//**
 *   @file   Communication.c
 *   @brief  Implementation of Communication Driver for Altera FPGAs.
 *   @author Istvan Csomortani (istvan.csomortani@analog.com)
********************************************************************************
 * Copyright 2012(c) Analog Devices, Inc.
 *
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *  - Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *  - Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *  - Neither the name of Analog Devices, Inc. nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *  - The use of this software may or may not infringe the patent rights
 *    of one or more patent holders.  This license does not release you
 *    from the requirement that you obtain separate licenses from these
 *    patent holders to use this software.
 *  - Use of the software either in source or binary form, must be run
 *    on or directly connected to an Analog Devices Inc. component.
 *
 * THIS SOFTWARE IS PROVIDED BY ANALOG DEVICES "AS IS" AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, NON-INFRINGEMENT,
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL ANALOG DEVICES BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, INTELLECTUAL PROPERTY RIGHTS, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
*******************************************************************************
 *   SVN Revision: $WCREV$
******************************************************************************/

/****************************************************************************/
/********************** Include Files ***************************************/
/****************************************************************************/
#include "Communication.h"

/****************************************************************************/
/********************* Functions Definitions ********************************/
/****************************************************************************/

/***************************************************************************//* *
 * @brief Initializes the SPI communication peripheral.
 *
 * @param lsbFirst - Transfer format (0 or 1).
 *                   Example: 0x0 - MSB first.
 *                            0x1 - LSB first.
 * @param clockFreq - SPI clock frequency (Hz).
 *                    The primary and secondary prescalers have to be modified
 *                    in order to change the SPI clock frequency
 * @param clockPol - SPI clock polarity (0 or 1).
 *                   Example: 0x0 - Idle state for clock is a low level; active
 *                                  state is a high level;
 *                        0x1 - Idle state for clock is a high level; active
 *                                  state is a low level.
 * @param clockEdg - SPI clock edge (0 or 1).
 *                   Example: 0x0 - Serial output data changes on transition
 *                                  from idle clock state to active clock
 *state;
 *                            0x1 - Serial output data changes on transition
 *                                  from active clock state to idle clock state
 *
 * @return status - Result of the initialization procedure.
 *                  Example: 1 - if initialization was successful;
 *                           0 - if initialization was unsuccessful.
 * NOTE : In case of the Altera SPI driver, the Init function just enables the
 * corresponding SPI core, by setting up the Control GPIO register, the
 * prototype was left 'untouched' for keeping the portability of the driver
******************************************************************************/
unsigned char SPI_Init(unsigned char lsbFirst,
                       unsigned long clockFreq,
                       unsigned char clockPol,
                       unsigned char clockEdg)
{
    IOWR_ALTERA_AVALON_PIO_DATA(CTRL_BASE, SPI_CFG);
    return 1;
}

/***************************************************************************//*
 * @brief Writes data to SPI.
 *
 * @param slaveDeviceId - The ID of the selected slave device.
 * @param data - Data represents the write buffer.
 * @param bytesNumber - Number of bytes to write.
 *
 * @return Number of written bytes.
******************************************************************************/

unsigned char SPI_Write(unsigned char slaveDeviceId,
                        unsigned char* data,
                        unsigned char bytesNumber)
{
    unsigned char index = 0;


    IORD_ALTERA_AVALON_SPI_RXDATA(SPI_BASE_ADDR);   // Discard any stale data, in case previous
                                                    // communication was interrupted.
    if(slaveDeviceId == 1)
    {
        IOWR_ALTERA_AVALON_SPI_SLAVE_SEL(SPI_BASE_ADDR, 1);
        IOWR_ALTERA_AVALON_SPI_CONTROL(SPI_BASE_ADDR, ALTERA_AVALON_SPI_CONTROL_SSO_MSK);
    }
    for(index = 0; index < bytesNumber; index++)
    {
        while((IORD_ALTERA_AVALON_SPI_STATUS(SPI_BASE_ADDR) &
               ALTERA_AVALON_SPI_STATUS_TRDY_MSK) == 0);            // Wait until txdata register is empty.
        IOWR_ALTERA_AVALON_SPI_TXDATA(SPI_BASE_ADDR, data[index]);      // Transmit data.
        while ((IORD_ALTERA_AVALON_SPI_STATUS(SPI_BASE_ADDR) &
                ALTERA_AVALON_SPI_STATUS_RRDY_MSK) == 0);           // Wait until rxdata register is full.
        //data[index] = IORD_ALTERA_AVALON_SPI_RXDATA(SPI_BASE_ADDR);   // Read data from rxdata register.
    }

    while((IORD_ALTERA_AVALON_SPI_STATUS(SPI_BASE_ADDR) &
           ALTERA_AVALON_SPI_STATUS_TMT_MSK) == 0);                 // Wait until the interface has finished transmitting.
    if(slaveDeviceId == 1)
    {
        IOWR_ALTERA_AVALON_SPI_CONTROL(SPI_BASE_ADDR, 0);
    }

    return bytesNumber;
}

/***************************************************************************//**
 * @brief Reads data from SPI.
 *
 * @param slaveDeviceId - The ID of the selected slave device.
 * @param data - Data represents the read buffer.
 * @param bytesNumber - Number of bytes to read.
 *
 * @return Number of read bytes.
*******************************************************************************/
unsigned char SPI_Read(unsigned char slaveDeviceId,
                       unsigned char* data,
                       unsigned char bytesNumber)
{
    unsigned char   index            = 0;


    IORD_ALTERA_AVALON_SPI_RXDATA(SPI_BASE_ADDR);   // Discard any stale data, in case previous
                                                    // communication was interrupted.
    if(slaveDeviceId == 1)
    {
        IOWR_ALTERA_AVALON_SPI_SLAVE_SEL(SPI_BASE_ADDR, 1);
        IOWR_ALTERA_AVALON_SPI_CONTROL(SPI_BASE_ADDR, ALTERA_AVALON_SPI_CONTROL_SSO_MSK);
    }
    for(index = 0; index < bytesNumber; index++)
    {
        while((IORD_ALTERA_AVALON_SPI_STATUS(SPI_BASE_ADDR) &
               ALTERA_AVALON_SPI_STATUS_TRDY_MSK) == 0);            // Wait until txdata register is empty.
        IOWR_ALTERA_AVALON_SPI_TXDATA(SPI_BASE_ADDR, data[index]);      // Transmit data.
        while ((IORD_ALTERA_AVALON_SPI_STATUS(SPI_BASE_ADDR) &
                ALTERA_AVALON_SPI_STATUS_RRDY_MSK) == 0);           // Wait until rxdata register is full.
        data[index] = IORD_ALTERA_AVALON_SPI_RXDATA(SPI_BASE_ADDR); // Read data from rxdata register.
    }

    while((IORD_ALTERA_AVALON_SPI_STATUS(SPI_BASE_ADDR) &
           ALTERA_AVALON_SPI_STATUS_TMT_MSK) == 0);                 // Wait until the interface has finished transmitting.
    if(slaveDeviceId == 1)
    {
        IOWR_ALTERA_AVALON_SPI_CONTROL(SPI_BASE_ADDR, 0);
    }
    /*
    IORD_ALTERA_AVALON_SPI_RXDATA(SPI_BASE_ADDR);                  // Discard the RX Buffer
    if(slaveDeviceId == 1)
    {
        IOWR_ALTERA_AVALON_SPI_SLAVE_SEL(SPI_BASE_ADDR, 1);            // Assert Slave Select
        IOWR_ALTERA_AVALON_SPI_CONTROL(SPI_BASE_ADDR, \
            ALTERA_AVALON_SPI_CONTROL_SSO_MSK);
    }
    for(index = 0; index < bytesNumber; index++)
    {
        while( (IORD_ALTERA_AVALON_SPI_STATUS(SPI_BASE_ADDR) & \
                ALTERA_AVALON_SPI_STATUS_TRDY_MSK) == 0 );       //Wait till transmission ends
        IOWR_ALTERA_AVALON_SPI_TXDATA(SPI_BASE_ADDR, data[index]);
        while( (IORD_ALTERA_AVALON_SPI_STATUS(SPI_BASE_ADDR) & \
                ALTERA_AVALON_SPI_STATUS_RRDY_MSK) == 0);
        data[index] = IORD_ALTERA_AVALON_SPI_RXDATA(SPI_BASE_ADDR);    // Save the content of the RX Buffer
    }
    while( (IORD_ALTERA_AVALON_SPI_STATUS(SPI_BASE_ADDR) & \
            ALTERA_AVALON_SPI_STATUS_TMT_MSK) == 0 );           //Wait till transmission ends
    if(slaveDeviceId == 1)
    {
        IOWR_ALTERA_AVALON_SPI_SLAVE_SEL(SPI_BASE_ADDR, 0);            // Deassert Slave Select
    }
    */

    return bytesNumber;
}

/**************************************************************************//**
 * @brief Initializes the UART communication peripheral.
 *
 * @param baudRate - Baud rate value.
 *                   Example: 9600 - 9600 bps.
 *
 * @return status - Result of the initialization procedure.
 *                  Example: 1 - if initialization was successful;
 *                           0 - if initialization was unsuccessful.
 *
******************************************************************************/

char UART_Init(unsigned long baudRate)
{
    //Clear the AC bit of the Control Register
    IOWR_ALTERA_AVALON_JTAG_UART_CONTROL(JTAG_UART_BASE, \
                                    ALTERA_AVALON_JTAG_UART_CONTROL_AC_MSK);
    return 1;
}

/**************************************************************************//**
 * @brief Writes one character to UART.
 *
 * @param data - Character to write.
 *
 * @return None.
******************************************************************************/
void UART_WriteChar(char data)
{
    while((IORD_ALTERA_AVALON_JTAG_UART_CONTROL(JTAG_UART_BASE) \
                & ALTERA_AVALON_JTAG_UART_CONTROL_WSPACE_MSK) == 0);
    IOWR_ALTERA_AVALON_JTAG_UART_DATA(JTAG_UART_BASE, data);
}

/***************************************************************************//**
 * @brief Reads one character from UART.
 *
 * @return receivedChar - Read character.
******************************************************************************/
void UART_ReadChar(char * data)
{
    unsigned int dataBuff = IORD_ALTERA_AVALON_JTAG_UART_DATA(JTAG_UART_BASE);

    while(!(dataBuff & ALTERA_AVALON_JTAG_UART_DATA_RVALID_MSK))    //data is valid
    {
        dataBuff = IORD_ALTERA_AVALON_JTAG_UART_DATA(JTAG_UART_BASE);
    }
    *data = (char)(dataBuff & ALTERA_AVALON_JTAG_UART_DATA_DATA_MSK) \
                            >> ALTERA_AVALON_JTAG_UART_DATA_DATA_OFST;
}

/**************************************************************************//**
 * @brief Writes one character to UART.
 *
 * @param data - Character to write.
 *
 * @return None.
******************************************************************************/
void UART_WriteString(const char* string)
{
    while(*string)
    {
        UART_WriteChar(*string++);
    }
}
